clear,clc
clear all
format short
load Datos.txt
load Coeficientes.txt
load Areas.txt
TProceso=cputime;
disp('                                              Viento Sinttico')
disp('                                   Universidad Nacional Autnoma de Mxico')
disp('                                     Nancy Alejandra Mosqueda Santacruz')

fprintf('\n\n')
disp('Programa para simular la accin dinmica del viento')
disp('Espere, por favor...')

Vo=Datos(1,1);
r=Datos(2,1);
w0=r/500; %Genera el intervalo de frecuencias a los que se analizar la 
%estructura, garantizando que la frecuencia fundamental se encuentre en el
%intervalo
dw=w0; %Representa el incremento en las frecuencias
%Cambio a velocidad promediada a 10 min
Uo=0.69*Vo;
noFrecuencias=round(2.5/dw);
wf=w0*noFrecuencias;
x=zeros(1,noFrecuencias);
w=linspace(w0 ,wf ,noFrecuencias);

%Funcin de densidad de turbulencia modificado de acuerdo a la norma
%canadiense

for i=1:noFrecuencias
    x(i)=1220*dw*i/Uo;
end
Sr=4*((x.^2)./((1+x.^2).^(4/3)))./w;

semilogx(w,Sr,'LineWidth',3)
legend('Espectro de Potencia de Davenport utilizado')
set(gca, 'fontsize', 10) 
xlabel('Frecuencias','Fontsize',10);
ylabel('Sr(f)');
whitebg('w')
grid on

% Esta parte define los coeficientes de arrastre previamente obtenidos para
% cada altura de la estructura
numCD=Datos(3,1);
% Se genera el vector de coeficientes de arrastre
CD=zeros(numCD,1);
CD=Coeficientes(:,1);
% Esta parte define las reas correspondientes a cada coeficiente de
% arrastre
areas=zeros(numCD,1);
areas=Areas(:,1);
h=Datos(4,1);
div=Datos(5,1);
%Por lo tanto lo que sigue es dividir h/hi para conocer la altura de cada tramo
hTramo=h/div;

%Esta parte es para ajustar el nmero de coeficientes de arrastre 
%con el nmero de secciones en las que se desea dividir la torre, de
%tal manera que el vector se distribuye uniformemente hasta llegar
%a cero. 
hT=zeros(numCD,1);
if numCD<11
    hT(1)=h;
    aux1=h/numCD;   
    for i=2:numCD
         hT(i)=hT(i-1,:)-aux1;
    end

else
    hT=zeros(numCD,1);
hT(1,:)=h;
for i=1:10
    hT(i+1,:)=  hT(i,:)-(hTramo/3);
end
aux1=hT(10)/(numCD-9);   
for i=11:numCD
   hT(i,:)=hT(i-1,:)-aux1;
end
end

%Descomposicin de presiones fluctuantes
armonicos=Datos(6,1);
k=zeros(armonicos,1);
for i=1
    k(i,:)=1;
end
for i=1:armonicos-1
    k(i+1,:)=k(i,:)+1;
end

%Armnico resonante en posicin 3 dado en este punto al relacionar 
%la siguiente funcin con los armnicos
R=3; 
fk=zeros(armonicos,1);
fk=r./(2.^(k-R));

%Perodos de la estructura, donde el perodo fundamental se encuentre en la
%tercera posicin como ya se haba resuelto para el armnico resonante
Tk=zeros(armonicos,1);
Tk=1./fk;

%Intervalo de frecuencias de armnicos para los que se integrar la funcin
%de densidad espectral
fak=zeros(armonicos,1);
fak=r./(2.^(k-0.5-R));

fpk=zeros(armonicos,1);
fpk=r./(2.^(k+0.5-R));

%Evaluando x respecto a los armnicos
xf=zeros(armonicos,1);
for i=1:armonicos
xf(i,:)=(1220*fk(i,:)/Uo);
end

%Funcin de densidad espectral
xf=xf.^2;
syms fk
SrF=5953600*fk/((1+1488400*fk^2/Uo^2)^(4/3)*Uo^2); %Simplificacin de la funcin

%Integral punto por punto
for i=1: armonicos
    ckSim(i)=sqrt(2*int(SrF,fpk(i),fak(i)));
    Ck(i)=double(ckSim(i));
end
Ck=Ck';
aux3=zeros(armonicos,1);
for i=1:armonicos
    aux3=aux3+Ck(i);
end
    
ck1=zeros(armonicos,1);
for i=1:armonicos
    ck1(i)=Ck(i)/aux3(i);
end

%ngulos de fase generados pseudoaleatoriamente...
theta=(2*pi).*rand(armonicos,1);

t=0:0.1:600; %Tiempo en incrementos de 0.1
% Tiempo total de la simulacin
T=6001;
P=zeros(T,armonicos);
for i=1:armonicos
    for j=1:T
        P(j,i)=cos((((2*pi)/(Tk(R)*2.^(k(i)-R))*t(j)-theta(i))));
    end
end

%Contribucin de componente resonante superestimada con un valor de 2, por
%lo que se realizala siguiente correcin
ck1(R)=ck1(R)/2;

%Garantizando que la suma de los coeficientes es unitaria, se realiza el
%siguiente ajuste
ck1(R-1)=ck1(R-1)+(ck1(R)/2);
ck1(R+1)=ck1(R+1)+(ck1(R)/2);

%Vector de Frecuencias 
f2=zeros(armonicos,1);
for i=1:armonicos
   f2(i)=1/Tk(i);
end

%Armnicos corregidos conforme al espectro del viento
Pp=zeros(6001,armonicos);

for i=1:armonicos
    for j=1:6001
        Pp(j,i)=P(j,i)*ck1(i);
    end
end

%Correlacin Espacial
%Tamao de rfaga equivalente
rafagaeq=zeros(armonicos,1);
for i=1:armonicos
    rafagaeq(i,:)=Uo/(7*f2(i,:));
end

%Correlacin final
Lk=zeros(armonicos,1);
for i=1:armonicos
    Lk(i,:)=2*rafagaeq(i,:);
end

%Posicin del centro de rfaga
CondG=Datos(11,1);
if CondG==0
Gc=h-(Lk(R)/2);
else
    Gc=CondG;
end

%Coeficiente de reduccin de presiones fluctuantes respecto a la altura de
%rfaga
Cred=zeros(numCD,armonicos);
for j=1:armonicos
   for k=1:numCD
      if Gc<=hT(k) && hT(k)<=Gc+rafagaeq(j)
            Cred(k,j)=(1/rafagaeq(j)).*(Gc-hT(k))+1;
      elseif Gc-rafagaeq(j)<=hT(k) && hT(k)<=Gc
        Cred(k,j)=(-1/rafagaeq(j))*(Gc-hT(k))+1;
      else  
        Cred(k,j)=0;
      end
   end
end

%Velocidad media
Vm=[];
b600=Datos(7,1);
p600=Datos(8,1);
for i=1:numCD
    Vm(:,:)=(0.69*b600*Vo)*(hT(:,:)/10).^p600;
end

%Velocidad Pico
Vp=[];
b3=Datos(9,1);
p3=Datos(10,1);
for i=1:numCD
    Vp(:,:)=b3*Vo*(hT(:,:)/10).^p3;
end

%Presin Pico
q=[];
for i=1:numCD
    q=0.613*(Vp).^2;
end

%*****Inicio Carga esttica *****
qest=[];
for i=1:numCD
    qest=0.613*Vm.^2;
end

Fest=[];
Fest=CD.*areas.*qest;
%*****Fin Carga esttica *****

%Presin fluctuante
qf=[];
qf=q-qest;
qf;

%Presin armnica variable en tiempo, altura y armnicos
datos=numCD*armonicos*6001;
Q=zeros(6001,numCD,armonicos);
FF=zeros(6001,numCD,armonicos);
pDin=zeros(6001,numCD);

for t=1:6001
    for j=1:numCD
        for k=1:armonicos
            Q(t,j,k)  = qf(j)*Cred(j,k)*Pp(t,k);
            FF(t,j,k) = CD(j)*areas(j)*Q(t,j,k);
            pDin(t,j) = pDin(t,j)+FF(t,j,k);
        end
    end
end

for i=1:numCD
    [fid]=fopen(sprintf('Viento%d.txt',i),'wt');
    fuerza=pDin(:,i);
    fprintf(fid,'\n%d',fuerza);
end

xlswrite('DatosSalida.xlsx',pDin(:,:),'FuerzasDinamicas','B2');
TiempoDeProceso=cputime-TProceso

disp('Fin del clculo, revisar el archivo "DatosSalida" de la misma carpeta')
